/*-- Copyright (c) 2016 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. --*/
/*
Model         	FAW 
Subject Area  	FAW 
Version			FAW 5.3 
Name          	ddcf_dimension_disaggregation : Script to disaggregate the forecast values to dimension lower level.
			    Script is wriiten for the Total and Average aggregation method. For the remaining aggregation method, bydefaut Total is used for diggregating the values.
				Script can be customized as per the need. 
				Used- 
				Forecast plan
				Collaboration plan:
Created by 		SAS R & D Pune, India.
Created on 		11/10/2016 
Last updated 	11/10/2016 

* Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513

Input Parameters:
plan_id = Unique Id of Forecast,
i_disagg_actuals_tab = Input table which contains the acuals at disaggregated level , 
i_agg_actuals_tab = , 
i_agg_predict_tab = ,
o_disagg_predict_tab =  ,
plan_kpi_aggregation_method = ,
kpi_var_nm = ,
agg_lvl_count = ,
disagg_lvl_count = ,
agg_hier_lvls_comma = ,
disagg_hier_lvls_comma = ,
plan_leaf_lvls_and_dim_leaf_lvls = , 
var_nm_of_actuals = ACTUAL ,
var_nm_of_predicts = PREDICTED ,
allow_negative_ind = 0
History:

*/

%macro ddcf_dimension_disaggregation( 	 plan_id = ,
										 i_disagg_actuals_tab = , 
										 i_agg_actuals_tab = , 
										 i_agg_predict_tab = ,
										 o_disagg_predict_tab =  ,
										 plan_kpi_aggregation_method = ,
										 kpi_var_nm = ,
										 agg_lvl_count = ,
										 disagg_lvl_count = ,
										 agg_hier_lvls_comma = ,
										 disagg_hier_lvls_comma = ,
										 plan_leaf_lvls_and_dim_leaf_lvls = , 
										 var_nm_of_actuals = ACTUAL ,
										 var_nm_of_predicts = PREDICTED ,
										 allow_negative_ind = 0

								   	);


%global dimension_disagg_err_ind ;
/*%global job_rc job_rc_msg;*/
%local disagg_hier_lvls_comma_space;
%local agg_hier_lvls_comma_space;
%local sql_kpi_aggregation_method;

%let job_rc = 0;
%let job_rc_msg = ; 

%let dimension_disagg_err_ind = 0;

data _null_;
	call symputx("disagg_hier_lvls_comma_space",tranwrd("&disagg_hier_lvls_comma.", ","," "));
	call symputx("agg_hier_lvls_comma_space",tranwrd("&agg_hier_lvls_comma.", ","," "));
run;


%if "%sysfunc(kupcase(&plan_kpi_aggregation_method.))" = "TOTAL" %then %do;
	%let sql_kpi_aggregation_method = SUM;
%end;
%else %if "%sysfunc(kupcase(&plan_kpi_aggregation_method.))" = "AVERAGE" %then %do;
	%let sql_kpi_aggregation_method = AVG;
%end;
%else %do;
	%let sql_kpi_aggregation_method = SUM;
%end;


%if "%sysfunc(kupcase(&plan_kpi_aggregation_method.))" = "TOTAL" or   
	"%sysfunc(kupcase(&plan_kpi_aggregation_method.))" = "AVERAGE"
%then %do;

		/* Acuals on plan leaf levels */
		proc sql noprint;
		    create table agg_lvl_actuals&plan_id. as
		    select &agg_hier_lvls_comma., 
		           &sql_kpi_aggregation_method.(&var_nm_of_actuals.) as agg_actual_value
		    from &i_agg_actuals_tab. 
		    group by &agg_hier_lvls_comma.
		    ;
		quit;

		%rcset(&sqlrc);
		%if (&job_rc>4) %then %do;
			%goto exit;
		%end;

		/* actuals on dimension leaf levels
		   Data of sum of KPI values at the given time series.
		   This is used to derive proportions of demand quantities. */
	
		proc sql noprint;
		    create table disagg_lvl_actuals&plan_id. as
		    select &plan_leaf_lvls_and_dim_leaf_lvls.,  
		           &sql_kpi_aggregation_method.(&kpi_var_nm.) as disagg_acual_value 
		    from &i_disagg_actuals_tab.
		    group by &plan_leaf_lvls_and_dim_leaf_lvls.  
			;
	    quit;

		%rcset(&sqlrc);
		%if (&job_rc>4) %then %do;
			%goto exit;
		%end; 

		/* Determines the proporation*/
	     data proportion_data&plan_id.;
		    merge agg_lvl_actuals&plan_id. disagg_lvl_actuals&plan_id.;
			by &agg_hier_lvls_comma_space.;
			if agg_actual_value ne 0 then do;
			   fraction_kpi_value = disagg_acual_value / agg_actual_value;
			end;
			else do;
				fraction_kpi_value = .;
			end;
	     run;
		
		 %rcset(&syserr);
		%if (&job_rc>4) %then %do;
			%goto exit;
		%end;

		/* Derive disaggregated forecasts. */
		proc sql noprint;
	       create table &o_disagg_predict_tab. as
		   select &plan_id. as fcst_plan_id,
				  %do i=1 %to &disagg_lvl_count.;
						%let rk_string = %sysfunc(kscan(%bquote(&disagg_hier_lvls_comma.),&i.));
						p.&rk_string., 
				  %end;
				  start_dt, 
				  f.&var_nm_of_predicts. * p.fraction_kpi_value as &var_nm_of_predicts.
	        from  &i_agg_predict_tab. f, proportion_data&plan_id. p
			where %do i=1 %to &agg_lvl_count.;
						%let rk_string = %sysfunc(kscan(%bquote(&agg_hier_lvls_comma.),&i.));
						f.&rk_string. =  p.&rk_string. and
				  %end;
			      f.&var_nm_of_predicts. IS NOT MISSING
			order by &disagg_hier_lvls_comma., start_dt
			  ;
	 	quit;

		%rcset(&sqlrc);
		%if (&job_rc>4) %then %do;
			%goto exit;
		%end;
%end;
/* %else %if <Condition> %then %do;
	%end;
*/
%else %do;
/* Sum is used as an aggregation method*/


/* Acuals on plan leaf levels */
		proc sql noprint;
		    create table agg_lvl_actuals&plan_id. as
		    select &agg_hier_lvls_comma., 
		           sum(&var_nm_of_actuals.) as agg_actual_value
		    from &i_agg_actuals_tab. 
		    group by &agg_hier_lvls_comma.
		    ;
		quit;

		%rcset(&sqlrc);
		%if (&job_rc>4) %then %do;
			%goto exit;
		%end;

		/* actuals on dimension leaf levels
		   Data of sum of KPI values at the given time series.
		   This is used to derive proportions of demand quantities. */
	
		proc sql noprint;
		    create table disagg_lvl_actuals&plan_id. as
		    select &plan_leaf_lvls_and_dim_leaf_lvls.,
		           sum(&kpi_var_nm.) as disagg_acual_value 
		    from &i_disagg_actuals_tab.
		    group by &plan_leaf_lvls_and_dim_leaf_lvls.
			;
	    quit;

		%rcset(&sqlrc);
		%if (&job_rc>4) %then %do;
			%goto exit;
		%end; 

		/* Determines the proporation*/
	     data proportion_data&plan_id.;
		    merge agg_lvl_actuals&plan_id. disagg_lvl_actuals&plan_id.;
			by &agg_hier_lvls_comma_space.;
			if agg_actual_value ne 0 then do;
			   fraction_kpi_value = disagg_acual_value / agg_actual_value;
			end;
			else do;
				fraction_kpi_value = .;
			end;
	     run;
		
		 %rcset(&syserr);
		%if (&job_rc>4) %then %do;
			%goto exit;
		%end;

		/* Derive disaggregated forecasts. */
		proc sql noprint;
	       create table &o_disagg_predict_tab. as
		   select &plan_id. as fcst_plan_id,
/*		   	      &kpi_rk. as kpi_rk,*/
				  %do i=1 %to &disagg_lvl_count.;
						%let rk_string = %sysfunc(kscan(%bquote(&disagg_hier_lvls_comma.),&i.));
						p.&rk_string., 
				  %end;
				  start_dt,
				  f.&var_nm_of_predicts. * p.fraction_kpi_value as &var_nm_of_predicts.
	        from  &i_agg_predict_tab. f, proportion_data&plan_id. p
			where %do i=1 %to &agg_lvl_count.;
						%let rk_string = %sysfunc(kscan(%bquote(&agg_hier_lvls_comma.),&i.));
						f.&rk_string. =  p.&rk_string. and
				  %end;
			      f.&var_nm_of_predicts. IS NOT MISSING
			order by &disagg_hier_lvls_comma., start_dt
			  ;
	 	quit;

		%rcset(&sqlrc);
		%if (&job_rc>4) %then %do;
			%goto exit;
		%end;
	
%end;


/*Validate that output table is exist */
%if %sysfunc(exist(&o_disagg_predict_tab.)) = 0 %then %do;
	%put ERROR: While disaggregating forecast values.;
	%let dimension_disagg_err_ind = 1;
	%goto exit;
%end;

%exit:

%mend;
